@@ -14,46 +14,48 @@ from pyglinet import utils
 import pyglinet.glinet_api as api_helper
 import pathlib
 import pickle
+from typing import Union, List, Dict
 
 
 class GlInet:
+    """
+    This class manages the connection to a GL-Inet router and provides basic routines to send and receive data.
+
+    Important: Only works with firmware version >=4.0. The api has changed from REST api to json-rpc with the 4.0,
+    so older firmware versions won't work.
+
+    Before you can start making requests, you need to call the :meth:`~pyglinet.GlInet.login` method
+
+    The api calls can either be made via the GlInetApi object,
+    which can be constructed via the get_api_client method, or via the request method directly.
+
+    :param url: url to router rpc api
+    :param username: username, default is root.
+    :param password: password, if left empty, a prompt will ask you when login() is called. For security reasons,
+        you should never pass your password here.
+    :param protocol_version: default 2.0
+    :param keep_alive: if set to True, a background thread will be started to keep the connection alive
+    :param keep_alive_intervall: intervall in which the background thread sends requests to the router
+    :param verify_ssl_certificate: either True/False or path to certificate.
+    :param update_api_reference_cache: if True, data is loaded from the web, otherwise application tries first to
+        load data from cache.
+    :param api_reference_url: url to api description
+    :param cache_folder: folder where data is persisted. If left empty, default is `$home/.python-pyglinet`
+    """
 
     def __init__(self,
-                 url="https://192.168.8.1/rpc",
-                 username="root",
-                 password=None,
-                 protocol_version="2.0",
-                 keep_alive=True,
-                 keep_alive_intervall=30,
-                 verify_ssl_certificate=False,
-                 update_api_reference_cache=False,
-                 api_reference_url="https://dev.gl-inet.cn/docs/api_docs_api/",
-                 cache_folder=None):
-        """
-        This class manages the connection to a GL-Inet router and provides basic routines to send and receive data.
-
-        Important: Only works with firmware version >=4.0. The api has changed from REST api to json-rpc with the 4.0,
-        so older firmware versions won't work.
-
-
-        The specific api calls are coordinated via the GlInetApi object,
-        which can be constructed via the get_api_client method.
-
-        :param url: url to router rpc api
-        :param username: username, default is root.
-        :param password: password, if left empty, a prompt will ask you when login() is called. For security reasons,
-        you should never pass your password here.
-        :param protocol_version: default 2.0
-        :param keep_alive: if set to True, a background thread will be started to keep the connection alive
-        :param keep_alive_intervall: intervall in which the background thread sends requests to the router
-        :param verify_ssl_certificate: either True/False or path to certificate.
-        :param update_api_reference_cache: if True, data is loaded from the web, otherwise application tries first to
-        load data from cache.
-        :param api_reference_url: url to api description
-        :param cache_folder: folder where data is persisted. If left empty, default is $home/.python-pyglinet
-        """
-        self.url = url
-        self.query_id = 0
+                 url: str = "https://192.168.8.1/rpc",
+                 username: str = "root",
+                 password: Union[str, None] = None,
+                 protocol_version: str = "2.0",
+                 keep_alive: bool = True,
+                 keep_alive_intervall: float = 30,
+                 verify_ssl_certificate: bool = False,
+                 update_api_reference_cache: bool = False,
+                 api_reference_url: str = "https://dev.gl-inet.cn/docs/api_docs_api/",
+                 cache_folder: bool = None):
+        self._url = url
+        self._query_id = 0
         if cache_folder is None:
             self._cache_folder = pathlib.Path.home()
             self._cache_folder = os.path.join(self._cache_folder, ".python-pyglinet")
@@ -65,10 +67,10 @@ class GlInet:
             else:
                 raise FileNotFoundError(f"Path {cache_folder} doesnt exist.")
         self._password = password
-        self.username = username
-        self.protocol_version = protocol_version
-        self.session = requests.session()
-        self.sid = None
+        self._username = username
+        self._protocol_version = protocol_version
+        self._session = requests.session()
+        self._sid = None
         self._keep_alive = keep_alive
         self._keep_alive_intervall = keep_alive_intervall
         self._thread = None
@@ -84,16 +86,16 @@ class GlInet:
         self._api_desciption = self.__load_api_desciption(update_api_reference_cache)
         self._keep_alive_interrupt_event = threading.Event()
 
-    def __generate_query_id(self):
+    def __generate_query_id(self) -> int:
         """
         Generate json-rpc query id
         :return: query id
         """
-        qid = self.query_id
-        self.query_id = (self.query_id + 1) % 9999999999
+        qid = self._query_id
+        self._query_id = (self._query_id + 1) % 9999999999
         return qid
 
-    def __generate_request(self, method, params):
+    def __generate_request(self, method: str, params: Union[Dict, List[str], str]) -> dict:
         """
         Generate json for rpc api call
         :param method: rpc method
@@ -101,41 +103,42 @@ class GlInet:
         :return: json
         """
         # if there was an successful login before
-        if self.sid:
+        if self._sid:
             if isinstance(params, dict):
-                params_ = {"sid": self.sid}
+                params_ = {"sid": self._sid}
                 params_.update(params)
                 return {
-                    "jsonrpc": self.protocol_version,
+                    "jsonrpc": self._protocol_version,
                     "id": self.__generate_query_id(),
                     "method": method,
                     "params": params_
                 }
             else:
                 return {
-                    "jsonrpc": self.protocol_version,
+                    "jsonrpc": self._protocol_version,
                     "id": self.__generate_query_id(),
                     "method": method,
-                    "params": [self.sid] + list(params)
+                    "params": [self._sid] + list(params)
                 }
         else:
             return {
-                "jsonrpc": self.protocol_version,
+                "jsonrpc": self._protocol_version,
                 "id": self.__generate_query_id(),
                 "method": method,
                 "params": params
             }
 
-    def request(self, method, params):
+    def request(self, method: str, params: Union[Dict, List[str], str]) -> utils.ResultContainer:
         """
         Send request to router
         :param method: rpc method
         :param params: parameter
+
         :return: result
         """
         req = self.__generate_request(method, params)
         self._lock.acquire()
-        resp = self.session.post(self.url, json=req, verify=False)
+        resp = self._session.post(self._url, json=req, verify=False)
         self._lock.release()
         if resp.json().get("error", None):
             error_ = resp.json().get("error")
@@ -157,6 +160,7 @@ class GlInet:
         :param json_data: json data
         :param method: api method call
         :param params: params
+
         :return: ResultContainer
         """
         if method == "call":
@@ -173,16 +177,16 @@ class GlInet:
         Requests required information to start login process.
         :return: challence
         """
-        resp = self.request("challenge", {"username": self.username})
+        resp = self.request("challenge", {"username": self._username})
         return resp.result
 
     @decorators.logout_required
     def login(self):
         """
-        Login
+        Login and start background thread for keep_alive is configured. If password was set in constructor,
+        cached values will be ignored. If password was not set (default) in :meth:`~pyglinet.GlInet`, you will be asked to enter the password
+        the first time this function is called. If login was successful, the password hash is cashed.
 
-        Login and start background thread for keep_alive is configured. If password was set,
-        cached values will be ignored.
         :return: True
         """
         challenge = self.__challenge_login()
@@ -197,9 +201,9 @@ class GlInet:
             # call challenge again since otherwise it will timeout
             challenge = self.__challenge_login()
             login_hash = self.__generate_login_hash(challenge)
-            resp = self.request("login", {"username": self.username,
+            resp = self.request("login", {"username": self._username,
                                           "hash": login_hash})
-            self.sid = resp.result.sid
+            self._sid = resp.result.sid
         except exceptions.AccessDeniedError:
             logging.warning("Could not login with current credentials, deleting cached credentials.")
             self._cached_login_data = None
@@ -230,7 +234,7 @@ class GlInet:
             password = getpass.getpass(prompt='Enter your GL-Inet password')
 
         _hash = self.__generate_unix_passwd_hash(password, challenge.alg, challenge.salt)
-        login_data = {"username": self.username,
+        login_data = {"username": self._username,
                       "hash": _hash,
                       "salt": challenge.salt,
                       "alg": challenge.alg}
@@ -238,11 +242,12 @@ class GlInet:
             self._cached_login_data = login_data
             self.__dump_to_file(self._cached_login_data, self._login_cache_path)
 
-    def __load_if_exist(self, file):
+    def __load_if_exist(self, file: str):
         """
         Load pickle file if it exists.
         :param file: path to file
-        :return: None if file doesnt exist, else Data
+
+        :return: None if file doesn't exist, else Data
         """
         loaded_data = None
         if os.path.exists(file):
@@ -256,18 +261,21 @@ class GlInet:
     def __dump_to_file(self, obj, file):
         """
         Dump pickle data to file.
+        :param obj: object to dump
         :param file: path to file
+
         :return: None
         """
         with open(file, "wb") as f:
             pickle.dump(obj, f)
 
-    def __keep_alive(self):
+    def __keep_alive(self) -> None:
         """
         Keep connection alive
 
         Function is started in background thread (see login() for more details). Send in fixed intervall requests to api.
         If not successful, try to connect again.
+
         :return: None
         """
         logging.info(f"Starting keep alive thread at intvervall {self._keep_alive_intervall}")
@@ -280,35 +288,38 @@ class GlInet:
         logging.info("Keep alive halted.")
 
     @decorators.login_required
-    def is_alive(self):
+    def is_alive(self) -> bool:
         """
         Check if connection is alive.
+
         :return: True if alive, else False
         """
-        if self.sid is None:
+        if self._sid is None:
             return False
         try:
-            resp = self.request("alive", {"sid": self.sid})
+            resp = self.request("alive", {"sid": self._sid})
         except exceptions.AccessDeniedError:
             return False
         return True
 
     @decorators.login_required
-    def logout(self):
+    def logout(self) -> bool:
         """
         Logout and stop keep alive thread
+
         :return: True
         """
-        self.request("logout", {"sid": self.sid})
-        self.sid = None
+        self.request("logout", {"sid": self._sid})
+        self._sid = None
         self._stop_keep_alive_thread()
         return True
 
-    def __generate_unix_passwd_hash(self, password, alg, salt):
+    def __generate_unix_passwd_hash(self, password: str, alg: str, salt: str) -> str:
         """
         Generate unix style hash with given algo and salt
         :param alg: algorithm
         :param salt: salt
+
         :return: hash
         """
         return crypt.crypt(password, f"${alg}${salt}")
@@ -317,16 +328,18 @@ class GlInet:
         """
         Generate final authentication hash
         :param challenge: dict with nonce, salt and algo type
+
         :return: authentication hash
         """
-        return hashlib.md5(f'{self.username}:{self._cached_login_data["hash"]}:{challenge.nonce}'.encode()).hexdigest()
+        return hashlib.md5(f'{self._username}:{self._cached_login_data["hash"]}:{challenge.nonce}'.encode()).hexdigest()
 
-    def __load_api_desciption(self, update=False):
+    def __load_api_desciption(self, update: bool = False):
         """
         Load api description in json format
 
         @:param update: if true, the api description is loaded from the web. If false, the program first tries to load
         the data from the cache and in case this fails from the web.
+
         :return: api description
         """
         api_description = None
@@ -348,15 +361,10 @@ class GlInet:
         self.request("call", "")
 
     @decorators.login_required
-    def get_api_client(self):
+    def get_api_client(self) -> api_helper.GlInetApi:
         """
-        Create client to access api functions
+        Create GlInetApi object client to access api functions
+
         :return: api client
         """
         return api_helper.GlInetApi(self._api_desciption, self)
-
-
-if __name__ == "__main__":
-    glinet = GlInet()
-    glinet.login()
-    api_client = glinet.get_api_client()
